import React, { useEffect, useState, useRef, useCallback } from "react";
import { Platform, } from "react-native";
import styled from "styled-components/native";
import { Flex, SlideIndicator } from "../index";
import { I18nManager } from "react-native";
const HorizontalScrollView = styled.ScrollView.attrs({ horizontal: true }) `
  flex: 1;
`;
/*
In RTL activated, this carousel is making a jump to the last item at start. It's patched by scrolling back to the activeIndex item, but i don't know how to patch this definitely.
It's seems like a behavior that react-native is doing to start from the end of list, except we don't want this kind of RTL behavior as this list is "time based" and need to always stay LTR.
Except a visual jump on RTL layout.
*/
function Carousel({ activeIndex = 0, autoDelay, restartAfterEnd = true, scrollOnSidePress = false, containerProps, slideIndicatorContainerProps, scrollViewProps, onChange, onOverflow, IndicatorComponent = SlideIndicator, maxDurationOfTap, children, onAutoChange, onManualChange, }) {
    const [init, setInit] = useState(false);
    const [activeIndexState, setActiveIndexState] = useState(activeIndex);
    const disableTimer = useRef(false);
    const [resetTimer, setResetTimer] = useState({});
    const dimensions = useRef(null);
    const slidesLength = React.Children.count(children);
    const scrollRef = useRef(null);
    const fullWidth = 100 * slidesLength;
    const itemWidth = !dimensions.current ? 0 : dimensions.current.contentWidth / slidesLength;
    const scrollToIndex = useCallback((index, animated = true) => {
        if (scrollRef.current && dimensions.current) {
            scrollRef.current.scrollTo({
                x: itemWidth * index,
                animated,
            });
        }
    }, [itemWidth]);
    useEffect(() => {
        // On init scroll to the active index prop location - if specified.
        if (init && typeof activeIndex === "number") {
            scrollToIndex(activeIndex, false);
        }
    }, [activeIndex, init, scrollToIndex]);
    useEffect(() => {
        if (scrollToIndex && typeof activeIndex === "number") {
            scrollToIndex(activeIndex, false);
        }
    }, [activeIndex, scrollToIndex]);
    const onContentSizeChange = (contentWidth, contentHeight) => {
        dimensions.current = { contentWidth, contentHeight };
        setInit(true);
    };
    const onTap = useCallback((event) => {
        const tapPositionXPercent = event.nativeEvent.locationX / itemWidth;
        if (tapPositionXPercent > 0.25) {
            if (slidesLength > activeIndexState + 1) {
                scrollToIndex(activeIndexState + 1, false);
                onManualChange && onManualChange(activeIndexState + 1);
            }
            else {
                onOverflow && onOverflow("end", false);
            }
        }
        else if (activeIndexState > 0) {
            scrollToIndex(activeIndexState - 1, false);
            onManualChange && onManualChange(activeIndexState - 1);
        }
        else {
            onOverflow && onOverflow("start", false);
        }
    }, [slidesLength, activeIndexState, scrollToIndex, onOverflow, itemWidth, onManualChange]);
    const onScroll = ({ nativeEvent: { contentOffset, contentSize }, }) => {
        const newIndex = Math.abs(Math.round((contentOffset.x / contentSize.width) * slidesLength));
        setActiveIndexState(newIndex);
        onChange && onChange(newIndex);
    };
    useEffect(() => {
        if (!autoDelay)
            return;
        const interval = setTimeout(() => {
            if (!disableTimer.current) {
                const newIndex = typeof activeIndexState !== "undefined" ? (activeIndexState + 1) % slidesLength : 0;
                onAutoChange && onAutoChange(newIndex);
                if (restartAfterEnd || newIndex !== 0) {
                    scrollToIndex(newIndex);
                }
                else {
                    onOverflow && onOverflow("end", true);
                }
            }
        }, autoDelay);
        return () => {
            return clearTimeout(interval);
        };
    }, [
        resetTimer,
        slidesLength,
        scrollToIndex,
        onChange,
        autoDelay,
        activeIndexState,
        onOverflow,
        restartAfterEnd,
        onAutoChange,
    ]);
    // Timestamp of start of click on the Carrousel
    const [tapTime, setTapTime] = useState(0);
    const onStartTap = useCallback(() => {
        setTapTime(new Date().getTime());
    }, []);
    const onEndTap = useCallback((event) => {
        const currentTime = new Date().getTime();
        if (!maxDurationOfTap || currentTime - tapTime <= maxDurationOfTap) {
            onTap(event);
        }
    }, [maxDurationOfTap, onTap, tapTime]);
    return (React.createElement(Flex, { flex: 1, width: "100%", alignItems: "center", justifyContent: "center", ...containerProps },
        React.createElement(HorizontalScrollView, { ref: scrollRef, onScroll: onScroll, onContentSizeChange: onContentSizeChange, onScrollBeginDrag: () => {
                disableTimer.current = true;
            }, onScrollEndDrag: () => {
                disableTimer.current = false;
            }, pagingEnabled: Platform.OS !== "web", showsHorizontalScrollIndicator: false, scrollEventThrottle: 200, contentContainerStyle: {
                width: `${fullWidth}%`,
                flexDirection: I18nManager.isRTL && Platform.OS !== "ios" ? "row-reverse" : "row",
            }, decelerationRate: "fast", onTouchStart: scrollOnSidePress ? onStartTap : undefined, onTouchEnd: scrollOnSidePress ? onEndTap : undefined, ...scrollViewProps }, React.Children.map(children, (child, index) => (React.createElement(Flex, { key: index, flex: 1 }, child)))),
        React.createElement(Flex, { my: 8, ...slideIndicatorContainerProps }, React.isValidElement(IndicatorComponent) ? (IndicatorComponent) : (
        /* @ts-expect-error TS 5 can't seem to be able to prove this is a react comopnent here */
        React.createElement(IndicatorComponent, { activeIndex: activeIndexState || 0, onChange: (index) => {
                scrollToIndex(index);
                setResetTimer({});
                onManualChange && onManualChange(index);
            }, slidesLength: slidesLength, duration: autoDelay })))));
}
export default React.memo(Carousel);
