import React from "react";
import { Dimensions, PixelRatio, Platform, StyleSheet } from "react-native";
import { CircledCheckSolidMedium } from "@ledgerhq/icons-ui/nativeLegacy";
import styled, { useTheme } from "styled-components/native";
import Svg, { Line } from "react-native-svg";
import Flex from "../../Flex";
const dashLength = 4;
const linesWidth = 2;
/**
 * We have to set a fixed size for the lines (and then they get clipped with overflow:hidden)
 * because their real height is dynamic (depends on the height of the step) and
 * there seems to be no way to have that height being dynamic without having the
 * scaling effect affecting the length of the stroke dash (the "- - - ").
 * This is kind of a trick but seemingly the only way to have a nice stroke dash
 **/
const linesLength = 5000;
const TopSegmentSvg = ({ status, hidden, height }) => {
    const theme = useTheme();
    const strokeColor = status === "inactive" ? theme.colors.neutral.c40 : theme.colors.primary.c80;
    const strokeDashArray = status === "inactive" ? `${dashLength}` : undefined;
    return (React.createElement(Flex, { height: height, width: "100%", overflow: "hidden" },
        React.createElement(Flex, { style: StyleSheet.absoluteFillObject, alignItems: "center" },
            React.createElement(Svg, { height: linesLength, width: linesWidth, viewBox: `0 0 ${linesWidth} ${linesLength}`, preserveAspectRatio: "xMinYMin slice" },
                React.createElement(Line, { key: status + " " + hidden, x1: "0", y1: "0", x2: "0", y2: "100", strokeWidth: hidden ? 0 : 2 * linesWidth, stroke: strokeColor, strokeDasharray: strokeDashArray })))));
};
const topSegmentDefaultHeight = Platform.OS === "android" ? 23 : 21; // difference due to how borders are drawn in android
const BottomSegmentSvg = ({ status, hidden }) => {
    const theme = useTheme();
    const strokeColor = status === "completed" ? theme.colors.primary.c80 : theme.colors.neutral.c40;
    const strokeDashArray = status === "completed" ? undefined : `${dashLength}`;
    return (React.createElement(Flex, { flex: 1, width: "100%", style: { transform: [{ scaleY: -1 }] }, overflow: "hidden" },
        React.createElement(Flex, { style: StyleSheet.absoluteFillObject, alignItems: "center" },
            React.createElement(Svg, { height: 2 * Dimensions.get("screen").height, width: linesWidth, viewBox: `0 0 ${linesWidth} ${linesLength}`, preserveAspectRatio: "xMinYMin slice" },
                React.createElement(Line, { key: status + " " + hidden, x1: "0", y1: "0", x2: "0", y2: "100%", strokeWidth: hidden ? 0 : 2 * linesWidth, stroke: strokeColor, strokeDasharray: strokeDashArray, strokeDashoffset: `${dashLength}` })))));
};
const getIconBackground = (theme, status, isLastItem) => {
    if (isLastItem) {
        if (status === "inactive")
            return theme.colors.success.c10;
        return "transparent";
    }
    else if (status === "active") {
        return theme.colors.neutral.c40;
    }
    else {
        return "transparent";
    }
};
const getIconBorder = (theme, status, isLastItem) => {
    if (isLastItem) {
        return theme.colors.success.c70;
    }
    else if (status === "inactive") {
        return theme.colors.neutral.c40;
    }
    return theme.colors.primary.c80;
};
const CenterCircle = styled(Flex) `
  border-radius: 9999px;
  width: 16px;
  height: 16px;
  background: ${(p) => getIconBackground(p.theme, p.status, p.isLastItem)};
  border: 2px solid ${(p) => getIconBorder(p.theme, p.status, p.isLastItem)};
  align-items: center;
  justify-content: center;
`;
export default function TimelineIndicator({ status, isFirstItem, isLastItem, topHeight, ...props }) {
    const { colors } = useTheme();
    return (React.createElement(Flex, { flexDirection: "column", alignItems: "center", ...props },
        React.createElement(TopSegmentSvg, { status: status, hidden: isFirstItem, height: PixelRatio.roundToNearestPixel(topHeight || topSegmentDefaultHeight) }),
        React.createElement(CenterCircle, { status: status, isLastItem: isLastItem }, status === "completed" && (React.createElement(CircledCheckSolidMedium, { color: isLastItem ? colors.success.c70 : colors.primary.c80, size: 20 }))),
        React.createElement(BottomSegmentSvg, { status: status, hidden: isLastItem })));
}
TimelineIndicator.topSegmentDefaultHeight = topSegmentDefaultHeight;
