import React, { useMemo } from "react";
import styled, { useTheme } from "styled-components/native";
import { space, color } from "styled-system";
import Text from "../../Text";
import CheckAlone from "@ledgerhq/icons-ui/nativeLegacy/CheckAloneMedium";
import CloseMedium from "@ledgerhq/icons-ui/nativeLegacy/CloseMedium";
import Animated, { useAnimatedStyle, withTiming, withDelay, Easing } from "react-native-reanimated";
const Separator = styled.View `
  flex: 1;
  height: 1px;
  background-color: ${(p) => p.theme.colors.neutral.c40};
  ${space}
`;
const SeparatorFilling = Animated.createAnimatedComponent(styled.View `
  height: 100%;
  background-color: ${(p) => p.theme.colors.neutral.c100};
`);
const AnimatedSeparator = ({ filled, delay, duration, ...spaceProps }) => {
    const animatedStyle = useAnimatedStyle(() => ({
        width: withDelay(delay, withTiming(filled ? "100%" : "0%", {
            duration: duration,
            easing: Easing.linear,
        })),
    }));
    return (React.createElement(Separator, { ...spaceProps },
        React.createElement(SeparatorFilling, { style: [animatedStyle] })));
};
// #endregion
// #region StepIcon
const StepIcon = {
    Container: styled.View `
    flex-direction: row;
    align-items: center;
    justify-content: center;
    ${space}
  `,
    Background: styled.View `
    justify-content: center;
    align-items: center;
    flex-direction: row;
    width: ${(p) => p.theme.space[5]}px;
    height: ${(p) => {
        /* the height of the background has to be odd, so we can correctly predict where the center pixel is
          and align the separators accordingly */
        const baseSpace = p.theme.space[5];
        return baseSpace % 2 === 0 ? baseSpace + 1 : baseSpace;
    }}px;
    border-radius: ${(p) => p.theme.radii[2]}px;
    overflow: hidden;
    margin-right: ${(p) => p.theme.space[1]}px;
    margin-left: ${(p) => p.theme.space[1]}px;
    ${color}
  `,
    Current: styled.View `
    width: ${(p) => p.theme.space[2]}px;
    height: ${(p) => p.theme.space[2]}px;
    border-radius: ${(p) => p.theme.radii[2]}px;
    background-color: ${(p) => p.theme.colors.primary.c90};
  `,
    Pending: styled.View `
    width: ${(p) => p.theme.space[1]}px;
    height: ${(p) => p.theme.space[1]}px;
    border-radius: ${(p) => p.theme.space[1]}px;
    background-color: ${(p) => p.theme.colors.neutral.c70};
  `,
    Completed: ({ color }) => React.createElement(CheckAlone, { size: 16, color: color }),
    Errored: ({ color }) => React.createElement(CloseMedium, { size: 16, color: color }),
};
// #endregion
// #region Step
const StepView = styled.View `
  align-items: center;
  justify-content: center;
`;
const ActiveText = styled(Text) `
  color: ${(p) => p.theme.colors.neutral.c100};
`;
const PendingText = styled(Text) `
  color: ${(p) => p.theme.colors.neutral.c70};
`;
const ErroredText = styled(Text) `
  color: ${(p) => p.theme.colors.error.c50};
`;
function Step({ state, label, showLeftSeparator, showRightSeparator, }) {
    const labelText = useMemo(() => {
        switch (state) {
            case "COMPLETED":
            case "CURRENT":
                return React.createElement(ActiveText, null, label);
            case "ERRORED":
                return React.createElement(ErroredText, null, label);
            case "PENDING":
                return React.createElement(PendingText, null, label);
        }
    }, [state, label]);
    const { colors } = useTheme();
    const icon = useMemo(() => {
        switch (state) {
            case "COMPLETED":
                return (React.createElement(StepIcon.Background, { backgroundColor: "primary.c20" },
                    React.createElement(StepIcon.Completed, { color: colors.primary.c90 })));
            case "CURRENT":
                return (React.createElement(StepIcon.Background, { backgroundColor: "primary.c20" },
                    React.createElement(StepIcon.Current, null)));
            case "ERRORED":
                return (React.createElement(StepIcon.Background, { backgroundColor: "warning.c30" },
                    React.createElement(StepIcon.Errored, { color: colors.error.c50 })));
            case "PENDING":
                return (React.createElement(StepIcon.Background, null,
                    React.createElement(StepIcon.Pending, null)));
        }
    }, [state, colors]);
    return (React.createElement(StepView, null,
        React.createElement(StepIcon.Container, { mb: 2 },
            showLeftSeparator && (React.createElement(AnimatedSeparator, { filled: state !== "PENDING", duration: 100, delay: state !== "PENDING" ? 350 : 0 })),
            icon,
            showRightSeparator && (React.createElement(AnimatedSeparator, { filled: state === "COMPLETED", duration: 100, delay: state === "COMPLETED" ? 0 : 350 }))),
        labelText));
}
const Container = styled.View `
  flex-direction: row;
  width: 100%;
`;
function Stepper({ steps, activeIndex, errored }) {
    const { space } = useTheme();
    const separatorMarginTop = useMemo(() => Math.floor(space[5] / 2), [space]);
    return (React.createElement(Container, null, steps.map((label, i) => (React.createElement(React.Fragment, { key: i },
        React.createElement(Step, { label: label, state: i === activeIndex
                ? errored
                    ? "ERRORED"
                    : "CURRENT"
                : i < (activeIndex ?? 0)
                    ? "COMPLETED"
                    : "PENDING", showLeftSeparator: i > 0, showRightSeparator: i < steps.length - 1 }),
        i < steps.length - 1 ? (React.createElement(AnimatedSeparator, { filled: i < (activeIndex ?? 0), duration: 250, delay: 100, mt: separatorMarginTop })) : null)))));
}
// #endregion
export default Stepper;
