import React, { useCallback, useState, useMemo } from "react";
import styled, { useTheme } from "styled-components/native";
import { ActivityIndicator, TouchableOpacity, View, } from "react-native";
import { buttonSizeStyle, getButtonColorStyle } from "../../cta/Button/getButtonStyle";
import { ctaIconSize, ctaTextType } from "../../cta/getCtaStyle";
import Text from "../../Text";
import { Icon as IconComponent } from "../../Icon";
import baseStyled from "../../styled";
const IconContainer = styled.View `
  ${(p) => p.iconButton ? "" : p.iconPosition === "left" ? `margin-right: 10px;` : `margin-left: 10px;`}
`;
export const Base = baseStyled(TouchableOpacity).attrs((p) => {
    // if onPressWhenDisabled prop exists then the button will look
    // disabled but will still be press-able.
    const disabled = !p.onPressWhenDisabled && p.disabled;
    const visuallyDisabled = p.onPressWhenDisabled && p.disabled;
    const onPress = visuallyDisabled ? p.onPressWhenDisabled : p.onPress;
    return {
        ...getButtonColorStyle(p.theme.colors, p).button,
        // Avoid conflict with styled-system's size property by nulling size and renaming it
        size: undefined,
        sizeVariant: p.size,
        disabled,
        onPress,
    };
}) `

  border-radius: ${(p) => p.theme.space[10]}px;
  padding: 0 ${(p) => p.theme.space[7]}px;
  flex-direction: row;
  border-style: solid;
  text-align: center;
  align-items: center;
  justify-content: center;
  align-content: center;
  overflow: hidden;
  position: relative;
  ${(p) => buttonSizeStyle[p.sizeVariant || "medium"]}

  ${(p) => (p.iconButton ? `padding: 0; width: ${p.theme.space[10]}px;` : "")}
`;
const Container = styled.View `
  flex-direction: row;
  text-align: center;
  align-items: center;
  justify-content: center;
  align-content: center;
  opacity: ${(p) => (p.hide ? 0 : 1)};
`;
const SpinnerContainer = styled.View `
  position: ${(p) => (p.displayContentWhenPending ? "relative" : "absolute")};
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  flex-direction: row;
  align-items: center;
  justify-content: center;
`;
const ButtonContainer = (props) => {
    const { Icon, isNewIcon, iconPosition = "right", children, hide = false, size = "medium", iconName, pending, displayContentWhenPending, buttonTestId, } = props;
    const theme = useTheme();
    const { text } = getButtonColorStyle(theme.colors, props);
    const IconNode = useMemo(() => {
        if (isNewIcon) {
            return Icon;
        }
        return ((iconName && React.createElement(IconComponent, { name: iconName, size: ctaIconSize[size], color: text.color })) ||
            (typeof Icon === "function" && React.createElement(Icon, { size: ctaIconSize[size], color: text.color })));
    }, [iconName, size, Icon, text.color, isNewIcon]);
    const textColor = useMemo(() => (pending ? theme.colors.neutral.c50 : text.color), [pending, theme.colors.neutral.c50, text.color]);
    return (React.createElement(Container, { hide: hide },
        iconPosition === "right" && children ? (React.createElement(Text, { testID: buttonTestId, variant: ctaTextType[size], fontWeight: "semiBold", color: textColor, ...(!Icon && { textAlign: "center" }) }, children)) : null,
        pending && displayContentWhenPending ? (React.createElement(IconContainer, { iconPosition: iconPosition },
            React.createElement(SpinnerContainer, { displayContentWhenPending: true },
                React.createElement(ActivityIndicator, { color: theme.colors.neutral.c50, animating: true })))) : IconNode ? (React.createElement(IconContainer, { iconButton: !children, iconPosition: iconPosition }, IconNode)) : null,
        iconPosition === "left" && children ? (React.createElement(Text, { variant: ctaTextType[size], fontWeight: "semiBold", color: textColor }, children)) : null));
};
const Button = (props) => {
    const { Icon, children, type = "default", iconName, disabled = false, pending = false, displayContentWhenPending = false, testID, } = props;
    const theme = useTheme();
    return (React.createElement(Base, { activeOpacity: 1, ...props, type: type, iconButton: (!!Icon || !!iconName) && !children, disabled: disabled || pending, testID: testID },
        React.createElement(View, null,
            React.createElement(ButtonContainer, { ...props, type: type, hide: pending && !displayContentWhenPending }),
            pending && !displayContentWhenPending ? (React.createElement(SpinnerContainer, { displayContentWhenPending: displayContentWhenPending },
                React.createElement(ActivityIndicator, { color: theme.colors.neutral.c50, animating: true }))) : null)));
};
export const PromisableButton = (props) => {
    const { Icon, children, onPress, type = "main", disabled = false, iconName } = props;
    const [spinnerOn, setSpinnerOn] = useState(false);
    const theme = useTheme();
    const onPressHandler = useCallback(async (event) => {
        if (!onPress)
            return;
        setSpinnerOn(true);
        try {
            await onPress(event);
        }
        finally {
            setSpinnerOn(false);
        }
    }, [onPress]);
    return (React.createElement(Base, { ...props, type: type, iconButton: (!!Icon || !!iconName) && !children, disabled: disabled || spinnerOn, onPress: onPressHandler },
        React.createElement(View, null,
            React.createElement(ButtonContainer, { ...props, type: type, hide: spinnerOn }),
            React.createElement(SpinnerContainer, null,
                React.createElement(ActivityIndicator, { color: theme.colors.neutral.c50, animating: spinnerOn })))));
};
export default Button;
